package RRPC;

import java.util.Vector;

public class AtomicProposition extends Proposition 
    implements java.io.Serializable {
	static final long serialVersionUID = 9157511079217862210L;
    private String name;

    public AtomicProposition (String _name){
	name = _name;
    }

    /**
     * from Sentence abstract class.
     *
     * Cannot replace AtomicProposition
     */
    public Sentence substitute(Substitution s) {
	return this;
    }

    public Sentence substituteVariable (Variable oldVar, Term newVar){
	return this;
    }

    public Sentence removeQuantifiers (Vector quantifications){
	return this;
    }

    // If <p> is an AtomicProposition and equals <this> then return an empty
    // substitution, otherwise return null.

    public Substitution unify (Proposition p, Substitution s) {
	if (p instanceof AtomicProposition &&
	    ((AtomicProposition)p).name.equals(name)) {
	    return s;
	} else {
	    return null;
	}
    } 

    // An atomic proposition subsumes itself and true
    public boolean subsumes (Proposition p, Substitution s){
	return ((p instanceof AtomicProposition &&
		 ((AtomicProposition)p).name.equals(name)) ||
		(p instanceof TrueProposition));
    }

    public Vector obtainVariables() {
	return new Vector();
    } 

    // Nothing to substitute
    public void substituteTerm(Term newTerm, Term oldTerm){
    }

    // No matches
    public Vector getPossibleMatches(Term t){
	return new Vector();
    }

    // We don't really need to copy the leaves
    public Proposition deepCopy(){
	return this;
    }

    public Proposition substituteTermAndCopy(Term newTerm, Term oldTerm){
	return this;
    }


    public String toString(){
	return name;
    }

} 
