
/**
 * CheckerFrame.java
 *
 * Title:			Resolution Refutation Proof Checker
 * Description:
 * @author			Leslie Kaelbling
 * @version
 */

package RRPC;

import java.io.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;


public class CheckerFrame extends javax.swing.JFrame {

    private Proof proof;
    private DefaultListModel axioms;
    private DefaultListModel steps;

    // We'll have two displayed lists.  One of axioms, one of proof steps
    javax.swing.JScrollPane axiomPane = new javax.swing.JScrollPane();
    javax.swing.JList axiomList = new javax.swing.JList();
    javax.swing.JScrollPane stepPane = new javax.swing.JScrollPane();
    javax.swing.JList stepList = new javax.swing.JList();
      
    // Make the buttons
    javax.swing.JButton addButton = new javax.swing.JButton("Add Axiom");
    javax.swing.JButton addFromFileButton = 
	new javax.swing.JButton("Load Axioms");
    javax.swing.JButton deleteButton = 
	new javax.swing.JButton("Delete Clause");
    javax.swing.JButton deleteStepButton = 
	new javax.swing.JButton("Delete Step");
    javax.swing.JButton conclusionButton = 
	new javax.swing.JButton("Add Conclusion");
    javax.swing.JButton clearProofButton = 
	new javax.swing.JButton("Clear Proof");
    javax.swing.JButton clearAxiomsButton = 
	new javax.swing.JButton("Clear Axioms");
    javax.swing.JButton proofButton = 
	new javax.swing.JButton("Start Proof");
    javax.swing.JButton resolveButton = 
	new javax.swing.JButton("Resolve");
    javax.swing.JButton factorButton = 
	new javax.swing.JButton("Factor");
    javax.swing.JButton paramodulateButton = 
	new javax.swing.JButton("Paramodulate");
    javax.swing.JButton loadButton = 
	new javax.swing.JButton("Load Proof");
    javax.swing.JButton saveButton = 
	new javax.swing.JButton("Save Proof");
    javax.swing.JButton outputButton = 
	new javax.swing.JButton("Output Proof");
    javax.swing.JButton quitButton = 
	new javax.swing.JButton("Quit");

    private JFileChooser choosy = new JFileChooser();

    //private Box menuBox;

    // Constructor
    public CheckerFrame() {

	// Make a proof
	proof = new Proof();
	axioms = proof.axioms();
	steps = proof.steps();

	// Hook the displayed lists up to the actual data vectors
	axiomList.setModel(axioms);
	stepList.setModel(steps);
        
	setTitle("Resolution Refutation Checker v0.9");
        
	// Quit on window close
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {System.exit(0);}
	    });
                
	// Make the content pane
	Box contentPane = javax.swing.Box.createHorizontalBox();
	setContentPane(contentPane);
            
	// At the top level, there are two boxes
	Box formulaBox = javax.swing.Box.createVerticalBox();
	contentPane.add(formulaBox);
	//menuBox = makeMenuBox();
	//contentPane.add(menuBox);
	contentPane.add(makeToolBar());
            
	// Formula box contains axioms and proof steps

	// Put the axioms and steps into a split pane
	JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
					      axiomPane, stepPane);
	splitPane.setOneTouchExpandable(true);
	splitPane.setDividerLocation(150);
	//Provide minimum sizes for the two components in the split pane
	Dimension minimumSize = new Dimension(100, 50);
	axiomPane.setMinimumSize(minimumSize);
	stepPane.setMinimumSize(minimumSize);

	stepPane.getViewport().add(stepList);
	stepList.setVisible(true);
	axiomPane.getViewport().add(axiomList);
	axiomList.setVisible(true);

	formulaBox.add(splitPane);
                
	pack();
    } 
        
    private JToolBar makeToolBar() {

	JToolBar toolBar = new JToolBar(JToolBar.VERTICAL);
	toolBar.setFloatable(false);

	toolBar.addSeparator();

	toolBar.add(addButton);
	toolBar.add(addFromFileButton);
	toolBar.add(conclusionButton);
	toolBar.add(deleteButton);
	toolBar.add(clearAxiomsButton);
	toolBar.add(proofButton);

	toolBar.addSeparator();

	toolBar.add(resolveButton);
	toolBar.add(factorButton);
	toolBar.add(paramodulateButton);
	toolBar.add(deleteStepButton);
	toolBar.add(clearProofButton);

	toolBar.addSeparator();

	toolBar.add(loadButton);
	toolBar.add(saveButton);
	toolBar.add(outputButton);
	toolBar.add(quitButton);
	toolBar.addSeparator();


	//Dimension buttonSize = new Dimension(300,20);
	//System.out.println(buttonSize);
	addButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//addButton.setMinimumSize(buttonSize);
	addFromFileButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//addFromFileButton.setPreferredSize(buttonSize);
	deleteButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//deleteButton.setPreferredSize(buttonSize);
	deleteStepButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//deleteStepButton.setPreferredSize(buttonSize);
	conclusionButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//conclusionButton.setPreferredSize(buttonSize);
	clearAxiomsButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//clearAxiomsButton.setPreferredSize(buttonSize);
	clearProofButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//clearProofButton.setPreferredSize(buttonSize);
	proofButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//proofButton.setPreferredSize(buttonSize);
	resolveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//resolveButton.setPreferredSize(buttonSize);
	factorButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//factorButton.setPreferredSize(buttonSize);
	paramodulateButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//paramodulateButton.setPreferredSize(buttonSize);
	loadButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//loadButton.setPreferredSize(buttonSize);
	saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//saveButton.setPreferredSize(buttonSize);
	outputButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//outputButton.setPreferredSize(buttonSize);
	quitButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	//quitButton.setPreferredSize(buttonSize);
            
	// Button listeners for the menu 
	// This is probably an archaic usage, but it works.
	addButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    addButtonActionPerformed(e);
		}
            });
	addFromFileButton.addActionListener
	    (new java.awt.event.ActionListener() {
		    public void actionPerformed(java.awt.event.ActionEvent e) {
			addFromFileButtonActionPerformed(e);
		}
            });
	deleteButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    deleteButtonActionPerformed(e);
		}
	    });
	deleteStepButton.addActionListener
	    (new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    deleteStepButtonActionPerformed(e);
		}
	    });
	deleteStepButton.setEnabled(false);

	conclusionButton.addActionListener(new java.awt.event.ActionListener(){
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    conclusionButtonActionPerformed(e);
		}
	    });
	clearAxiomsButton.addActionListener
	    (new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    clearAxiomsButtonActionPerformed(e);
		}
	    });

	clearProofButton.addActionListener
	    (new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    clearProofButtonActionPerformed(e);
		}
	    });
	clearProofButton.setEnabled(false);

	proofButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    proofButtonActionPerformed(e);
		}
	    });
	resolveButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    resolveButtonActionPerformed(e);
		}
	    });
	resolveButton.setEnabled(false);

	factorButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    factorButtonActionPerformed(e);
		}
	    });
	factorButton.setEnabled(false);

	paramodulateButton.addActionListener
	    (new java.awt.event.ActionListener() {
		    public void actionPerformed(java.awt.event.ActionEvent e) {
			paramodulateButtonActionPerformed(e);
		    }
		});
	paramodulateButton.setEnabled(false);

	loadButton.addActionListener
	    (new java.awt.event.ActionListener() {
		    public void actionPerformed(java.awt.event.ActionEvent e) {
			loadButtonActionPerformed(e);
		    }
		});


	saveButton.addActionListener
	    (new java.awt.event.ActionListener() {
		    public void actionPerformed(java.awt.event.ActionEvent e) {
			saveButtonActionPerformed(e);
		    }
		});

	outputButton.addActionListener
	    (new java.awt.event.ActionListener() {
		    public void actionPerformed(java.awt.event.ActionEvent e) {
			outputButtonActionPerformed(e);
		    }
		});

	quitButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    quitButtonActionPerformed(e);
		}
	    });

	return toolBar;
        
    } // end of make tool bar

    public void addButtonActionPerformed(java.awt.event.ActionEvent e) {

	// Create a new window and get the axiom
	String input = 
	    JOptionPane.showInputDialog (this, 
					 "Please enter an axiom.",
					 "Axiom Input", 
					 JOptionPane.PLAIN_MESSAGE);
	if (input != null){
	    try {
		// Read in a sentence
		Sentence s = Parser.startSentence(new Tokenizer
						  (new StringReader(input)));
		// Make it into clauses and add them to our list of axioms
		addAxioms(s.clausalForm());

	    } catch (IOException ioe) {
		JOptionPane.showMessageDialog(this, 
					      ioe.getMessage(),
					      "IO Exception", 
					      JOptionPane.ERROR_MESSAGE);
	    } catch (RuntimeException re) {
		JOptionPane.showMessageDialog(this, 
					      re.getMessage(),
					      "Runtime Exception", 
					      JOptionPane.ERROR_MESSAGE);
	    }
	}
    }


    // Add these axioms to our proof
    private void addAxioms (ClauseList newClauses){
	for (int i = 0; i < newClauses.size(); i++){
	    // Rename the variables again, right before we add it to the Vector
	    proof.addAxiom(newClauses.clauseAt(i).renameVariables());
	}
    }

    public void addFromFileButtonActionPerformed(java.awt.event.ActionEvent e){

	// Ask the user to choose a file
	int returnVal = choosy.showOpenDialog(this);

	if (returnVal != JFileChooser.APPROVE_OPTION) {
	    // User bailed out
	    return;
	}
	 
	// User selected a file
	File file = choosy.getSelectedFile();

	try {
	    Tokenizer toots = new Tokenizer(file.getPath());
	    
	    Sentence s = Parser.startSentence(toots);
	    addAxioms(s.clausalForm());
	    while (toots.hasMoreTokens()){
		s = Parser.parseSentence(toots);
		addAxioms(s.clausalForm());
	    }
	}
	catch (IOException ioe) {
	    JOptionPane.showMessageDialog(this, 
					  ioe.getMessage(),
					  "IO Exception", 
					  JOptionPane.ERROR_MESSAGE);
	} catch (RuntimeException re) {
	    JOptionPane.showMessageDialog(this, 
					  re.getMessage(),
					  "Runtime Exception", 
					  JOptionPane.ERROR_MESSAGE);
	}
    }

    // If the proof is started, we can delete proof steps
    // If not, we can delete axioms
    private void deleteButtonActionPerformed(java.awt.event.ActionEvent e) {
	int[] indices = axiomList.getSelectedIndices();
	if (indices.length != 1){
	    JOptionPane.showMessageDialog
		(this, 
		 "Please select one axiom to delete.");
	} else {
	    proof.deleteAxiomAt(indices[0]);
	}
    }


    private void deleteStepButtonActionPerformed
	(java.awt.event.ActionEvent e){
	int[] indices = stepList.getSelectedIndices();
	if (indices.length != 1){
	    JOptionPane.showMessageDialog
		(this, 
		 "Please select one proof step to delete.");
	} else {
	    proof.deleteStepAt(indices[0]);
	}
    }

    // Get the user to enter a formula.  Negate it, get the clauses, 
    // add them to the list
    public void conclusionButtonActionPerformed(java.awt.event.ActionEvent e) {
	// Create a new window and get the axiom
	String input = JOptionPane.showInputDialog
	    (this, 
	     "Please enter the desired conclusion.",
	     "Conclusion Input", 
	     JOptionPane.PLAIN_MESSAGE);

	if (input != null){
	    try {
		// Read in a sentence
		Sentence s = new Negation(Parser.startSentence
					  (new Tokenizer
					   (new StringReader(input))));
		// Make it into clauses and add them to our list of clauses
		addAxioms(s.clausalForm());

	    } catch (IOException ioe) {
		JOptionPane.showMessageDialog(this, 
					      ioe.getMessage(),
					      "IO Exception", 
					      JOptionPane.ERROR_MESSAGE);
	    } catch (RuntimeException re) {
		JOptionPane.showMessageDialog(this, 
					      re.getMessage(),
					      "Runtime Exception", 
					      JOptionPane.ERROR_MESSAGE);
	    }
	}
    }

    // Clear everything
    public void clearAxiomsButtonActionPerformed
	(java.awt.event.ActionEvent e) {
	proof.clear();
	// In case we are coming from proof mode, reset the buttons
	axiomModeButtons();
    }

    // Just clear out the current proof steps
    // Go back to axiom mode
    public void clearProofButtonActionPerformed
	(java.awt.event.ActionEvent e) {
	proof.restart();
	axiomModeButtons();
    }

    private void axiomModeButtons(){
	addButton.setEnabled(true);
	addFromFileButton.setEnabled(true);
	deleteButton.setEnabled(true);
	deleteStepButton.setEnabled(false);
	conclusionButton.setEnabled(true);
	clearAxiomsButton.setEnabled(true);
	clearProofButton.setEnabled(false);
	proofButton.setEnabled(true);
	factorButton.setEnabled(false);
	paramodulateButton.setEnabled(false);
	resolveButton.setEnabled(false);
    } 

    // Go to proof mode
    public void proofButtonActionPerformed(java.awt.event.ActionEvent e) {
	proof.start();
	proofModeButtons();
    }

    private void proofModeButtons(){
	// disable appropriate buttons
	addButton.setEnabled(false);
	addFromFileButton.setEnabled(false);
	deleteButton.setEnabled(false);
	deleteStepButton.setEnabled(true);
	conclusionButton.setEnabled(false);
	clearAxiomsButton.setEnabled(false);
	clearProofButton.setEnabled(true);
	proofButton.setEnabled(false);
	factorButton.setEnabled(true);
	paramodulateButton.setEnabled(true);
	resolveButton.setEnabled(true);
	//menuBox.repaint();
    }

    // Do a proof step.  Offer the user the choice of multiple results.

    public void resolveButtonActionPerformed(java.awt.event.ActionEvent e){
	int[] axiomIndices = axiomList.getSelectedIndices();
	int[] stepIndices = stepList.getSelectedIndices();
	if (axiomIndices.length + stepIndices.length != 2) {
	    JOptionPane.showMessageDialog
		(this, 
		 "Please select two clauses to resolve.");
	    return;
	}
	
	// two clauses are selected for resolution.
	ProofLine p1;
	ProofLine p2;
	if (axiomIndices.length == 0){
	    p1 = (ProofLine)steps.elementAt(stepIndices[0]);
	    p2 = (ProofLine)steps.elementAt(stepIndices[1]);
	} else if (axiomIndices.length == 1){
	    p1 = (ProofLine)steps.elementAt(stepIndices[0]);
	    p2 = (ProofLine)axioms.elementAt(axiomIndices[0]);
	} else {
	    p1 = (ProofLine)axioms.elementAt(axiomIndices[0]);
	    p2 = (ProofLine)axioms.elementAt(axiomIndices[1]);
	}

	Clause c1 = p1.clause();
	Clause c2 = p2.clause();

	Vector results = Resolver.resolveMultipleReturnSubstitutions(c1, c2);
	ClauseList resultClauses = (ClauseList) results.elementAt(0);
	Vector substitutions = (Vector) results.elementAt(1);
	if (resultClauses.size() == 0){
	    JOptionPane.showMessageDialog
		(this, 
		 "These two clauses cannot be resolved.");
	} else  if (resultClauses.size() == 1){
	    // Add the single result
	    proof.addProofStep(resultClauses.clauseAt(0),
			       ProofStep.RESOLUTION,
			       p1, p2, 
			       (Substitution) substitutions.elementAt(0));

	} else {
	    // Show the user all the clauses and let them pick one
	    Vector resultVector = resultClauses._clauses();

	    // Now, put it in a dialog box
	    ObjectListDialog.initialize(this, 
					    resultVector, 
					    "Choose Resolution Result",
		    "Please select one of the following possible results:");
	    Clause selectedClause = 
		(Clause) ObjectListDialog.showDialog(null);

	    if (selectedClause != null) {
		proof.addProofStep
		    (selectedClause,
		     ProofStep.RESOLUTION,
		     p1, p2, 
		     ((Substitution) substitutions.elementAt
				   (resultVector.indexOf(selectedClause))));
	    }

	}
	// Set things up for next time
	axiomList.clearSelection();
	stepList.clearSelection();
    }


    // There may be too many options for paramodulation given two clauses
    // Help the user select the pieces

    public void paramodulateButtonActionPerformed
	(java.awt.event.ActionEvent e){

	int[] axiomIndices = axiomList.getSelectedIndices();
	int[] stepIndices = stepList.getSelectedIndices();
	if (axiomIndices.length + stepIndices.length != 2) {
	    JOptionPane.showMessageDialog
		(this, 
		 "Please select two clauses to paramodulate.");
	    return;
	}
	
	// two clauses are selected for resolution.
	ProofLine p1;
	ProofLine p2;
	if (axiomIndices.length == 0){
	    p1 = (ProofLine)steps.elementAt(stepIndices[0]);
	    p2 = (ProofLine)steps.elementAt(stepIndices[1]);
	} else if (axiomIndices.length == 1){
	    p1 = (ProofLine)steps.elementAt(stepIndices[0]);
	    p2 = (ProofLine)axioms.elementAt(axiomIndices[0]);
	} else {
	    p1 = (ProofLine)axioms.elementAt(axiomIndices[0]);
	    p2 = (ProofLine)axioms.elementAt(axiomIndices[1]);
	}

	Clause c1 = p1.clause();
	Clause c2 = p2.clause();
	
	// find all the Equals predicates and ask the user to select one
	Vector equalsLiterals0 = c1.extractEqualsLiterals();
	Vector equalsLiterals1 = c2.extractEqualsLiterals();

	// Put them together into a single vector
	Vector equalsLiterals = new Vector(equalsLiterals0);
	equalsLiterals.addAll(equalsLiterals1);

	if (equalsLiterals.size() == 0){
	    JOptionPane.showMessageDialog
		(this, 
		 "We can only do paramodulation if there's an equation " +  
		 "in one of the clauses");
	    return;
	}

	Literal selectedEquality;
	if (equalsLiterals.size() == 1){
	    // There's only one equality, so just use it
	    selectedEquality = 
		(Literal) equalsLiterals.elementAt(0);
	} else {
	    // There are multiple equalities, so let the user pick
	    ObjectListDialog.initialize(this, 
					equalsLiterals,
					"Choose Equation",
            "Please select one of the following equations:");

	    selectedEquality = 
		(Literal) ObjectListDialog.showDialog(null);
	}
	
	// Now choose which term from the equality is going to get 
	// matched to a term in the other clause
	CompoundProposition cp = 
	    (CompoundProposition)selectedEquality.proposition();
	Vector terms = new Vector();
	Term t1 = cp.getTerms().getFirst();
	terms.addElement(t1);
	Term t2 = cp.getTerms().getRest().getFirst();
	terms.addElement(t2);
	ObjectListDialog.initialize(this, terms,
				    "Choose Term",
				    "Please select a term to match against a term from the other clause");
	Term matchTerm = (Term) ObjectListDialog.showDialog(null);
	Term substituteTerm = ((matchTerm == t1) ? t2 : t1);
	
	// Now, name the clauses so we know which one had the 
	// equality and which didn't
	Clause clauseWithEquation;
	Clause clauseWithoutEquation;
	if (equalsLiterals0.contains(selectedEquality)){
	    clauseWithEquation = c1;
	    clauseWithoutEquation = c2;
	} else {
	    clauseWithEquation = c2;
	    clauseWithoutEquation =c1;
	}
				    
	// Figure out which terms from the other clause possibly match
	// the user's selected matchTerm
	Vector possibleMatches =
	    clauseWithoutEquation.getPossibleMatches(matchTerm);
	
	if (possibleMatches.size() == 0){
	    JOptionPane.showMessageDialog
		(this, 
		 "We can only do paramodulation if there's a matching " +  
		 "term in the other clause");
	    return;
	}

	Term matchedTerm = null;
	if (possibleMatches.size() == 1){
	    // There's only one match, so just use it
	    matchedTerm =
		(Term) possibleMatches.elementAt(0);
	} else {
	    // There are multiple matches, so let the user pick
	    ObjectListDialog.initialize(this, possibleMatches,
					"Choose Matching Term",
			    "The following terms are possible matches for " 
				    + matchTerm + ".  Please choose one.");
	    matchedTerm = (Term) ObjectListDialog.showDialog(null);
	}

	// Now, we substitute substituteTerm in for matchedTerm, drop
	// the equation, concatenate the rest of the clauses, and apply
	// the unifying substitution (between matchTerm and matchedTerm)
	    
	// Start by copying the clauses
	// Need to do a deep copy, because we're about to mangle
	// the terms
	Clause cWith = new Clause(clauseWithEquation);
	// Remove the equation (has to be from a shallow copy or it
	// won't match)
	cWith.removeLiteral(selectedEquality);
	Clause cWithout = new Clause(clauseWithoutEquation);

	// Substitute (no good now, because equality won't do)
	// We should probably do substitute and copy at the same
	// time
	Clause cWithoutSubst = 
	    cWithout.substituteTermAndCopy(substituteTerm, matchedTerm);
	
	// Get the unifier
	Substitution s = matchTerm.unify(matchedTerm, new Substitution());
	// Apply it to everyone
	Clause result = cWith.concatenate(cWithoutSubst).substitute(s);

	// Add result to proof.  Whew!
	proof.addProofStep(result,
			   ProofStep.PARAMODULATION,
			   p1, p2, s);

	axiomList.clearSelection();
	stepList.clearSelection();
    }

    // Factor a clause
    public void factorButtonActionPerformed(java.awt.event.ActionEvent e) {
	int[] axiomIndices = axiomList.getSelectedIndices();
	int[] stepIndices = stepList.getSelectedIndices();
	if (axiomIndices.length + stepIndices.length != 1) {
	    JOptionPane.showMessageDialog
		(this, 
		 "Please select one clause for factoring.");
	    return;
	}
	
	// one clause is selected
	ProofLine p;
	if (axiomIndices.length == 0){
	    p = (ProofLine)steps.elementAt(stepIndices[0]);
	} else {
	    p = (ProofLine)axioms.elementAt(axiomIndices[0]);
	}

	Clause c = p.clause();
	Clause newC = c.factor();

	if (newC == null){
	    JOptionPane.showMessageDialog
		(this,
		 "This clause cannot be factored.");
	} else {
	    proof.addProofStep(newC,
			       ProofStep.FACTORING,
			       p, null, new Substitution());
	}
	axiomList.clearSelection();
	stepList.clearSelection();
    }

    public void loadButtonActionPerformed(java.awt.event.ActionEvent e) {
	// Ask user for a file name
	int returnVal = choosy.showDialog(this, "Load");
	if (returnVal != JFileChooser.APPROVE_OPTION) {
	    // User bailed out
	    return;
	}
	// User selected a file
	File file = choosy.getSelectedFile();

	try {
	    // Open it
	    ObjectInputStream ois = 
	    new ObjectInputStream(new FileInputStream(file));
	    // Read proof
	    proof = (Proof) ois.readObject();
	    // Connect it up to our windows
	    axioms = proof.axioms();
	    steps = proof.steps();
	    axiomList.setModel(axioms);
	    stepList.setModel(steps);
	    // Get the buttons in the right mode
	    if (proof.isStarted()){
		proofModeButtons();
	    } else {
		axiomModeButtons();
	    }

	    // Close file
	    ois.close();
	}
	catch (IOException ioe) {
	    JOptionPane.showMessageDialog(this, 
					  ioe.getMessage(),
					  "IO Exception", 
					  JOptionPane.ERROR_MESSAGE);
	} catch (RuntimeException re) {
	    JOptionPane.showMessageDialog(this, 
					  re.getMessage(),
					  "Runtime Exception", 
					  JOptionPane.ERROR_MESSAGE);
	} catch (ClassNotFoundException cnfe){
	    JOptionPane.showMessageDialog(this, 
					  cnfe.getMessage(),
					  "Class not found", 
					  JOptionPane.ERROR_MESSAGE);
	}
    }


    public void saveButtonActionPerformed(java.awt.event.ActionEvent e) {
	// Ask user for a file name
	int returnVal = choosy.showSaveDialog(this);
	if (returnVal != JFileChooser.APPROVE_OPTION) {
	    // User bailed out
	    return;
	}
	// User selected a file
	File file = choosy.getSelectedFile();

	try {
	    // Open it
	    ObjectOutputStream oos = 
		new ObjectOutputStream(new FileOutputStream(file));
	    // Save the current proof
	    oos.writeObject(proof);
	    // Close file
	    oos.close();
	}
	catch (IOException ioe) {
	    JOptionPane.showMessageDialog(this, 
					  ioe.getMessage(),
					  "IO Exception", 
					  JOptionPane.ERROR_MESSAGE);
	} catch (RuntimeException re) {
	    JOptionPane.showMessageDialog(this, 
					  re.getMessage(),
					  "Runtime Exception",
					  JOptionPane.ERROR_MESSAGE);
	}
    }

    // Save in a text file
    public void outputButtonActionPerformed(java.awt.event.ActionEvent e) {
	// Ask user for a file name
	int returnVal = choosy.showSaveDialog(this);
	if (returnVal != JFileChooser.APPROVE_OPTION) {
	    // User bailed out
	    return;
	}
	// User selected a file
	File file = choosy.getSelectedFile();

	try {
	    // Open it
	    PrintStream ps = new PrintStream(new FileOutputStream(file));
	    // Save the current proof
	    for (int i = 0; i < axioms.size(); i++){
		ps.println(axioms.elementAt(i));
	    }
	    for (int i = 0; i < steps.size(); i++){
		ps.println(steps.elementAt(i));
	    }

	    // Close file
	    ps.close();
	}
	catch (IOException ioe) {
	    JOptionPane.showMessageDialog(this, 
					  ioe.getMessage(),
					  "IO Exception", 
					  JOptionPane.ERROR_MESSAGE);
	} catch (RuntimeException re) {
	    JOptionPane.showMessageDialog(this, 
					  re.getMessage(),
					  "Runtime Exception",
					  JOptionPane.ERROR_MESSAGE);
	}
    }


    // Quit!
    public void quitButtonActionPerformed(java.awt.event.ActionEvent e) {
	// Quit the applicaton
	System.exit(0);
    }

}

