package RRPC;

import java.util.Vector;

public class CompoundProposition extends Proposition 
    implements java.io.Serializable{
	static final long serialVersionUID = -1482215837534573509L;
    private Symbol _relationSymbol;
    private TermList _terms;


    public CompoundProposition (Symbol relationSymbol, TermList terms) {
	_relationSymbol = relationSymbol;
	_terms = terms;
    }


    public Symbol getRelationSymbol() {
	return _relationSymbol;
    }

    public TermList getTerms() {
	return _terms;
    }


    /**
     * implemented from Sentence abstract class
     *
     */
    public Sentence substitute(Substitution s) {
	return new CompoundProposition(_relationSymbol,
				       _terms.substitute(s));
    }

    /**
     * implemented from interface Proposition
     */
    public Sentence substituteVariable(Variable oldVar, Term newVar){
	return new CompoundProposition
	    (_relationSymbol,
	     _terms.substituteVariable(oldVar, newVar));
    }

    /**
     * implemented from Proposition interface
     *
     * convert free variables to constants
     */
    public Sentence removeQuantifiers(Vector quantifications){
	return new CompoundProposition 
	    (_relationSymbol,
	     _terms.removeQuantifiers(quantifications));
    }

    /**
     * implemented from Proposition interface
     *
     * if <p> is a CompoundProposition, the propositional symbols are equal, 
     * and the terms are unifiable, then return the substitution, else 
     * return null.
     */
    public Substitution unify(Proposition p, Substitution s) {

	if (p instanceof CompoundProposition &&
	    ((CompoundProposition)p)._relationSymbol.equals(_relationSymbol) &&
	    ((CompoundProposition)p)._terms.length() == _terms.length()) {
	    // proposition and arity match, so resolve term lists
	    return _terms.unify(((CompoundProposition)p)._terms, s);
	} else {
	    return null;
	}
    } 

    // Does this proposition subsume (or equal) p?  
    // This is like one-sided unification.  p(x) subsumes p(C)
    // It also subsumes True
    // But p(x) does not subsume p(y)

    // Most of this is vestigial and is now handled by factoring

    public boolean subsumes (Proposition p, Substitution s) {
	return (p instanceof TrueProposition);

	//	return 
	//    (p instanceof TrueProposition ||
	//     (p instanceof CompoundProposition &&
	//      ((CompoundProposition)p)._relationSymbol.equals(_relationSymbol) &&
	//	      ((CompoundProposition)p)._terms.length() == _terms.length() &&
	//	      _terms.subsume(((CompoundProposition)p)._terms, s)));
    }
	

    /**
     * from Proposition abstract class.
     *
     * get the variables in this proposition.
     */
    public Vector obtainVariables() {
	return _terms.obtainVariables();
    }

    // Substitute newTerm in for oldTerm wherever it occurs
    // term1 might be compound or a constant (not necessarily a var)
    // Okay to check for object equality with oldTerm
    public void substituteTerm(Term newTerm, Term oldTerm){
	_terms.substituteTerm(newTerm, oldTerm);
    }

    // Return a vector of terms contained in this proposition that unify with t
    public Vector getPossibleMatches(Term t){
	return _terms.getPossibleMatches(t);
    }

    // We need to copy this structure and the term list, but not the symbol
    public Proposition deepCopy(){
	return new CompoundProposition(_relationSymbol, _terms.deepCopy());
    }

    public Proposition substituteTermAndCopy(Term newTerm, Term oldTerm){
	return new CompoundProposition
	    (_relationSymbol,
	     _terms.substituteTermAndCopy(newTerm, oldTerm));
    }

    public String toString() {
	return _relationSymbol + "(" +
	    ((_terms == null) ? "" : _terms.toString()) + ")";
    } 


} 
