package RRPC;

import java.util.Vector;

public class Conjunction extends Sentence {

	private Sentence s1;
	private Sentence s2;

	public Conjunction (Sentence _s1, Sentence _s2){
		s1 = _s1;
		s2 = _s2;
	}

	public Sentence eliminateEquivalences (){
		return new Conjunction (s1.eliminateEquivalences(),
						        s2.eliminateEquivalences());
	}

	public Sentence eliminateImplications (){
		return new Conjunction (s1.eliminateImplications(),
								s2.eliminateImplications());
	}

	//  Return the negation of this sentence by applying deMorgan's law
	public Sentence negate (){
		return new Disjunction (s1.negate(),
								s2.negate());
	}

	// Return an equivalent sentence
	public Sentence driveInNegations(){
		return new Conjunction (s1.driveInNegations(),
								s2.driveInNegations());
	}

  /**
   * from Sentence abstract class
   */
  public Sentence substitute(Substitution s) {
    return new Conjunction (s1.substitute(s), s2.substitute(s));
  } // end of METHHOD substitute()

	// Substitute variable new for variable old throughout
	public Sentence substituteVariable (Variable oldVar, Term newVar){
		return new Conjunction (s1.substituteVariable(oldVar, newVar),
							    s2.substituteVariable(oldVar, newVar));
	}

	// Rename any variables that are quantified within the components
	// of the conjunction
	public Sentence renameVariables (){
		return new Conjunction (s1.renameVariables(),
								s2.renameVariables());
	}

	// Extract the quantifications and add them to the vector
	public void extractQuantifications (Vector quantifications){
		s1.extractQuantifications(quantifications);
		s2.extractQuantifications(quantifications);
	}

	// Return the sentence minus the quantifiers
	public Sentence removeQuantifiers (Vector quantifications){
		return new Conjunction (s1.removeQuantifiers(quantifications),
								s2.removeQuantifiers(quantifications));
	}

	// Return a list of clauses
	// Conjunction is easy:  just concatenate the clause lists of the
	// two conjuncts

	public ClauseList makeClauses (){
		return s1.makeClauses().concatenate(s2.makeClauses());
	}




	public String toString(){
		return "(" + s1 + " ^ " + s2 + ")";
	}


}