package RRPC;

import java.util.Vector;

public class Disjunction extends Sentence {

	private Sentence s1;
	private Sentence s2;

	public Disjunction (Sentence _s1, Sentence _s2){
		s1 = _s1;
		s2 = _s2;
	}

	public Sentence eliminateEquivalences (){
		return new Disjunction(s1.eliminateEquivalences(),
							   s2.eliminateEquivalences());
	}

	public Sentence eliminateImplications (){
		return new Disjunction(s1.eliminateImplications(),
							   s2.eliminateImplications());
	}

	// Return the negation of this sentence by applying deMorgan's law
	public Sentence negate (){
		return new Conjunction(s1.negate(),
							   s2.negate());
	}

	// Return an equivalent sentence with negations driven in
	public Sentence driveInNegations(){
		return new Disjunction(s1.driveInNegations(),
							   s2.driveInNegations());
	}

  /**
   * from Sentence abstract class
   */
  public Sentence substitute(Substitution s) {
    return new Disjunction(s1.substitute(s), s2.substitute(s));
  } // end of METHOD substitute

	public Sentence substituteVariable (Variable oldVar, Term newVar){
		return new Disjunction(s1.substituteVariable(oldVar, newVar),
							   s2.substituteVariable(oldVar, newVar));
	}

	public Sentence renameVariables(){
		return new Disjunction(s1.renameVariables(),
							   s2.renameVariables());
	}

	// Extract the quantifications and add them to the vector
	public void extractQuantifications (Vector quantifications){
		s1.extractQuantifications(quantifications);
		s2.extractQuantifications(quantifications);
	}

	public Sentence removeQuantifiers (Vector quantifications){
		return new Disjunction(s1.removeQuantifiers(quantifications),
							   s2.removeQuantifiers(quantifications));
	}

	// This one's a bit tricky.  If s1 converts to (a v b) ^ (c v d) and
	// s2 converts to (e v f) ^ (g v h) then we need to generate
	// (a v b v e v f) ^ (a v b v g v h) ^ (c v d v e v f) ^ (c v d v g v h)

	public ClauseList makeClauses(){
		ClauseList c1 = s1.makeClauses();
		ClauseList c2 = s2.makeClauses();

		ClauseList result = new ClauseList();

		for (int i = 0; i < c1.size(); i++){
			for (int j = 0; j < c2.size(); j++){
				result.addClause( c1.clauseAt(i).
						  concatenate(c2.clauseAt(j))); 
			}
		}

		return result;
	}






	public String toString(){
		return "(" + s1 + " v " + s2 + ")";
	}


}
