package RRPC;

import java.util.Vector;

public class Function extends Term implements java.io.Serializable{
	static final long serialVersionUID = 5012269658279360651L;
    /////
    // VARIABLES
    private Symbol _functionSymbol;
    private TermList _terms;

    /////
    // CONSTRUCTOR
    public Function (Symbol functionSymbol, TermList terms) {
	_functionSymbol = functionSymbol;
	_terms = terms;
    }

    /////
    // ACCESSOR METHODS
    public Symbol getFunctionSymbol() {
	return _functionSymbol;
    } // end of METHOD getFunctionSymbol()

    public TermList getTerms() {
	return _terms;
    } // end of METHOD getTerms()


    public Substitution unify(Term t, Substitution s) {
	// if <t> is a Variable, call Variable's unify()
	if (t instanceof Variable)
	    return ((Variable)t).unify(this, s);
	// first check to make sure <t> is a function
	else if (t instanceof Function) {
	    // check the first terms
	    Function tf = (Function)t;
	    if (!_functionSymbol.equals(tf._functionSymbol)) {
		return null;
	    }

	    // term list of <t>
	    TermList ttl = ((Function)t).getTerms();
	    // term list of <this>
	    TermList thistl = _terms;

	    // check to make sure they have the same number of terms
	    // if not, they cannot unify
	    if (ttl.length() != thistl.length())
		return null;

	    // Unify the term lists
	    return thistl.unify(ttl, s);
	}
	else {
	    return null;
	}
    }


    // Like unify, but don't swap direction

    public Substitution subsumes (Term t, Substitution s) {
	// first check to make sure <t> is a function
	if (t instanceof Function) {
	    // check the first terms
	    Function tf = (Function)t;
	    if (!_functionSymbol.equals(tf._functionSymbol)) {
		return null;
	    }

	    // term list of <t>
	    TermList ttl = ((Function)t).getTerms();
	    // term list of <this>
	    TermList thistl = _terms;

	    // check to make sure they have the same number of terms
	    // if not, they cannot unify
	    if (ttl.length() != thistl.length())
		return null;

	    // Unify the term lists
	    return thistl.unify(ttl, s);
	}
	else {
	    return null;
	}
    }


    public Term substituteVariable(Variable oldVar, Term newVar){
	return new Function (_functionSymbol,
			     _terms.substituteVariable(oldVar, newVar));
    } // end of METHOD substituteVariable()

    /**
     * from Term abstract class
     *
     * only the terms of the function are of concern.  Returns a new Function
     * object with appropriate terms replaced
     */
    public Term substitute(Substitution s) {
	return new Function (_functionSymbol,
			     _terms.substitute(s));
    } // end of METHOD substitute()

    /**
     * from Term abstract class
     *
     * retrieves all the Variables and returns them in a Vector.
     */
    public Vector obtainVariables() {
	return _terms.obtainVariables();
    } // end of METHOD obtainVariables()

    // Convert free vars to contants
    public Term removeQuantifiers (Vector quantifications){
	return new Function (_functionSymbol,
			     _terms.removeQuantifiers(quantifications));
    }

    public void substituteTerm(Term newTerm, Term oldTerm){
	_terms.substituteTerm(newTerm, oldTerm);
    }

    // Return a vector of terms contained in this function that unify with t
    public Vector getPossibleMatches(Term t){
	return _terms.getPossibleMatches(t);
    }

    public Term deepCopy(){
	return new Function(_functionSymbol,
			    _terms.deepCopy());
    }

    public Term substituteTermAndCopy(Term newTerm, Term oldTerm){
	return new Function(_functionSymbol,
			    _terms.substituteTermAndCopy(newTerm, oldTerm));
    }

    public String toString(){
	return _functionSymbol + "(" + ((_terms==null)?"":_terms.toString())
	    + ")";
    }

}
