package RRPC;

import javax.swing.*;

public class Proof implements java.io.Serializable {
	static final long serialVersionUID = -8096882685555665135L;
    // Logically, axioms and steps would be vectors.  But we need to hook
    // them up to mutable lists in the UI, and, for some reason, Vectors
    // don't implement the interface ListModel.  So, we'll use
    // DefaultListModel instead.

    private DefaultListModel axioms;
    private DefaultListModel steps;
    private boolean started;

    public Proof() {
	axioms = new DefaultListModel();
	steps = new DefaultListModel();
	started = false;
    }

    public DefaultListModel axioms(){
	return axioms;
    }

    public DefaultListModel steps(){
	return steps;
    }

    public void addAxiom(Clause clause){
	if (started){
	    JOptionPane.showMessageDialog
		(null, 
		 "You cannot add axioms to a proof that has been started.");
	} else {
	    axioms.addElement(new Axiom(clause, axioms.size()+1));
	}
    }

    // Delete this axiom and renumber
    // Can only be done when the proof isn't started
    public void deleteAxiomAt(int i){
	axioms.removeElementAt(i);
	renumber();
    }

    // Remove all the proof steps that depend on this one
    public void deleteStepAt(int i){
	deleteStepAtAux(i);
	renumber();
    }

    // Do the recursion here so we only renumber once
    private void deleteStepAtAux(int i){
	// This won't change my index because they all have to come later
	deleteDependencies((ProofStep) steps.elementAt(i));
	steps.removeElementAt(i);
    }

    // Delete all the lines that depend on this line (and so, recursively)
    private void deleteDependencies(ProofStep step){
	// Could be more efficient
	int i = 0;
	while (i < steps.size()){
	    ProofStep currentStep = (ProofStep) steps.elementAt(i);
	    if (currentStep.dependsOn(step)){
		deleteStepAtAux(i);
		// Go around again, in case the new i is also a dependency
	    } else {
		i++;
	    }
	}
    }

    // Renumber all axioms and steps
    private void renumber(){
	for (int i = 0; i < axioms.size(); i++){
	    ((ProofLine) axioms.elementAt(i)).setLineNumber(i+1);
	}
	for (int i = 0; i < steps.size(); i++){
	    ((ProofLine) steps.elementAt(i)).setLineNumber(axioms.size()+i+1);
	}
    }

    public void start(){
	    started = true;
    }

    public boolean isStarted(){
	return started;
    }

    // Be sure the variables are new
    public void addProofStep(Clause clause, int rule, ProofLine d1, 
			     ProofLine d2, Substitution s){
	steps.addElement(new ProofStep(clause.renameVariables(), 
				       axioms.size() + steps.size() + 1,
				       rule, d1, d2, s));
	if (clause.isEmpty()){
	    JOptionPane.showMessageDialog(null, "QED!");
	}
    }

    // Clear all proof steps
    public void restart(){
	steps.clear();
	started = false;
    }

    // Clear whole proof
    public void clear(){
	axioms.clear();
	steps.clear();
	started = false;
    }

    // Return axiom i
    public Axiom axiomAt(int i){
	return (Axiom) axioms.elementAt(i);
    }

    // Return number of axioms
    public int numAxioms(){
	return axioms.size();
    }

    // Return proof line i
    public ProofLine proofLineAt(int i){
	if (i <= numAxioms()){
	    return (ProofLine) axioms.elementAt(i);
	} else {
	    return (ProofLine) steps.elementAt(i - numAxioms());
	}
    }
}
