package RRPC;

public class ProofStep extends ProofLine implements java.io.Serializable {
    static final long serialVersionUID = -6110086974080137101L;
    private int rule;
    private ProofLine derivedFrom1;
    private ProofLine derivedFrom2;
    private Substitution unifier;

    // Constants indicating what rule was used to derive this result.
    public static final int RESOLUTION = 0;
    public static final int FACTORING = 1;
    public static final int PARAMODULATION = 2;

    public ProofStep (Clause new_clause, int new_lineNumber,
		      int new_rule, ProofLine new_derivedFrom1, 
		      ProofLine new_derivedFrom2, 
		      Substitution new_unifier){
	clause = new_clause;
	lineNumber = new_lineNumber;
	rule = new_rule;
	derivedFrom1 = new_derivedFrom1;
	derivedFrom2 = new_derivedFrom2; // Set to null for factoring
	unifier = new_unifier;
    }
		      
    public boolean isAxiom(){
	return false;
    }

    // Return true if this step depends on step
    public boolean dependsOn(ProofStep step){
	return (step == derivedFrom1 || step == derivedFrom2);
    }

    // Return a string indicating which rule was used to 
    // derive this step
    private String ruleString(){
	if (rule == RESOLUTION){
	    return "Res";
	} else if (rule == FACTORING){
	    return "Fact";
	} else if (rule == PARAMODULATION){
	    return "Para";
	} else {
	    return "Ugly Internal Error";
	}
    }

    public String toString(){
	return "S" + lineNumber + ": " + clause +
	    "      " + ruleString() + "(" + derivedFrom1.lineNumber() + 
	    ((derivedFrom2 == null)? "" 
	     : ("," + derivedFrom2.lineNumber())) + ") " +
	    unifier;
    }

}
