package RRPC;

import java.util.Vector;

abstract public class Proposition extends Sentence 
    implements java.io.Serializable {
	static final long serialVersionUID = 8623758515538503975L;
    public Sentence eliminateEquivalences(){
	return this;
    }

    public Sentence eliminateImplications(){
	return this;
    }

    public Sentence driveInNegations(){
	return this;
    }

    // We can't negate this internally, so just make a negation
    public Sentence negate(){
	return new Negation(this);
    }

    public abstract Sentence substituteVariable (Variable oldVar, Term newVar);

    // This only applies at the level of quantifiers.  No new vars to rename
    // inside a proposition.
    public Sentence renameVariables(){
	return this;
    }

    // No quantifiers to extract here
    public void extractQuantifications (Vector quantifications){}

    // Get all the Variables in this proposition.  If there are none, return
    // an empty Vector.
    public abstract Vector obtainVariables();

    // Take this opportunity to turn free vars into constants
    public abstract Sentence removeQuantifiers (Vector quantifications);

    // Try to unify <this> with <p>
    public Substitution unify(Proposition p){
	return unify(p, new Substitution());
    }

    // Assume substitution s is already given to us
    public abstract Substitution unify (Proposition p, Substitution s);

    // Return true if this subsumes p
    public boolean subsumes(Proposition p){
	return subsumes(p, new Substitution());
    }
    
    // Assume substitution s is already given to us
    public abstract boolean subsumes(Proposition p, Substitution s);

    public ClauseList makeClauses(){
	return new ClauseList(new PositiveLiteral(this));
    }

    // Substitute newTerm in for oldTerm wherever it occurs
    // term1 might be compound or a constant (not necessarily a var)
    // Okay to check for object equality with oldTerm
    public abstract void substituteTerm(Term newTerm, Term oldTerm);

    // Return a vector of terms contained in this proposition that unify with t
    public abstract Vector getPossibleMatches(Term t);

    public abstract Proposition deepCopy();

    public abstract Proposition 
	substituteTermAndCopy(Term newTerm, Term oldTerm);

}
