/**
 * Resolve.java
 *
 * Title:			Resolution Refutation Proof Checker
 * Description:
 * @author			Leslie Kaelbling
 * @version
 */

package RRPC;

import java.util.Vector;

public class Resolver {

    public static Clause resolve(Clause c1, Clause c2){

	int c1size = c1.size();
	int c2size = c2.size();
	
	// Try to unify all pairs of literals from the two clauses
	// Right now, just return the first result we get;;
	// Eventually, return a list of possible resolutions

	// Need to remember when we score
	int c1winner = -1;
	int c2winner = -1;
	Substitution s = null;
	
	for (int c1i = 0; c1i < c1size; c1i++) {
	    for (int c2i = 0; c2i < c2size; c2i++) {
		s = unify(c1.literalAt(c1i), c2.literalAt(c2i));
		if (s != null){
		    // We have a winner
		    // Copy the clauses
		    Clause c1c = new Clause(c1);
		    Clause c2c = new Clause(c2);
		    // Remove the resolved literals 
		    c1c.removeLiteralAt(c1i);
		    c2c.removeLiteralAt(c2i);	
		    // Put the clauses together, apply s, and return
		    return (c1c.concatenate(c2c)).substitute(s);
		}
	    }
	}

	// Complete failure
	return null;
    }

    // Return a list of possible resolution results

    public static ClauseList resolveMultiple(Clause c1, Clause c2){

	int c1size = c1.size();
	int c2size = c2.size();
	
	// Try to unify all pairs of literals from the two clauses
	// Right now, just return the first result we get;;
	// Eventually, return a list of possible resolutions

	// Need to remember when we score
	int c1winner = -1;
	int c2winner = -1;
	Substitution s = null;
	ClauseList result = new ClauseList();
	
	for (int c1i = 0; c1i < c1size; c1i++) {
	    for (int c2i = 0; c2i < c2size; c2i++) {
		s = unify(c1.literalAt(c1i), c2.literalAt(c2i));
		if (s != null){
		    // We have a winner
		    // Copy the clauses
		    Clause c1c = new Clause(c1);
		    Clause c2c = new Clause(c2);
		    // Remove the resolved literals 
		    c1c.removeLiteralAt(c1i);
		    c2c.removeLiteralAt(c2i);	
		    // Add to the list of results
		    result.addClause((c1c.concatenate(c2c)).substitute(s));
		}
	    }
	}
	return result;
    }

    // Return a vector with two items:
    // 0.  A ClauseList of possible resolution results
    // 1.  A vector of substitutions (one for each res result)

    public static Vector 
	resolveMultipleReturnSubstitutions(Clause c1, Clause c2){

	int c1size = c1.size();
	int c2size = c2.size();
	
	// Try to unify all pairs of literals from the two clauses
	// Right now, just return the first result we get;;
	// Eventually, return a list of possible resolutions

	// Need to remember when we score
	int c1winner = -1;
	int c2winner = -1;
	Substitution s = null;
	ClauseList result = new ClauseList();
	Vector substitutions = new Vector();
	
	for (int c1i = 0; c1i < c1size; c1i++) {
	    for (int c2i = 0; c2i < c2size; c2i++) {
		s = unify(c1.literalAt(c1i), c2.literalAt(c2i));
		if (s != null){
		    // We have a winner
		    // Copy the clauses
		    Clause c1c = new Clause(c1);
		    Clause c2c = new Clause(c2);
		    // Remove the resolved literals 
		    c1c.removeLiteralAt(c1i);
		    c2c.removeLiteralAt(c2i);	
		    // Add to the list of results
		    Clause resultClause = (c1c.concatenate(c2c)).substitute(s);
		    // Should check to be sure we don't already have
		    // an alphabetic variant of this clause
		    result.addClause(resultClause);
		    substitutions.addElement(s);
		}
	    }
	}
	
	// Cons up the return value
	Vector returnValue = new Vector();
	returnValue.addElement(result);
	returnValue.addElement(substitutions);

	return returnValue;
    }




    public static Substitution unify(Literal l1, Literal l2){
	
	Substitution s = null;

	if ((l1.isPositive() && l2.isNegative()) ||
	    (l1.isNegative() && l2.isPositive())) {
	    // One is positive and one is negative, so try to unify 
	    // the propositions
	    s = l1.proposition().unify(l2.proposition());
	}
	return s;
    }

}

