package RRPC;

import java.util.Vector;

abstract public class Sentence {

    static private boolean _debugPrint = true;

    abstract public Sentence eliminateEquivalences();

    abstract public Sentence eliminateImplications();

    // Drive in negations to get an equivalent sentence
    abstract public Sentence driveInNegations();

    // Negate this sentence by driving the negation in
    abstract public Sentence negate();

    public ClauseList clausalForm(){
	Sentence processedSentence =
	    eliminateEquivalences().
	    eliminateImplications().
	    driveInNegations().
	    renameVariables();

	// Move the quantifiers to the front.  Also change
	// free vars to constants
	Vector quantifications = new Vector();
	processedSentence.extractQuantifications(quantifications);
	Sentence body = processedSentence.removeQuantifiers(quantifications);

	// Skolemize
	body = skolemize(quantifications, body);

	// Put into clausal form
	return body.makeClauses();

    }

    abstract public Sentence substitute(Substitution s);

    abstract public Sentence substituteVariable (Variable oldVar, Term newVar);

    abstract public Sentence renameVariables();

    abstract public void extractQuantifications (Vector quantifications);

    // Remove the quantifiers *and* convert free variables to constants
    abstract public Sentence removeQuantifiers(Vector quantifications);

    abstract public ClauseList makeClauses();

    private Sentence skolemize (Vector quantifications, Sentence body){
	// Walk through the quantifications, looking for existentials;
	// substitute skolem functions into the body

	for (int i = 0; i < quantifications.size(); i++){
	    Quantification qi = (Quantification) quantifications.elementAt(i);
	    if (qi.isExistential()){

		// Make a list of all the universally quantified
		// variables outside this scope
		TermList terms = null;
		for (int j = 0; j < i; j++){
		    Quantification qj = 
			(Quantification) quantifications.elementAt(j);
		    if (qj.isUniversal()){
			terms = new TermList(qj.variable(),terms);
		    }
		}

		// Make the skolem function and substitute it in
		if (terms == null){
		    // Just a constant
		    body = body.substituteVariable
			(qi.variable(),
			 new ConstantTerm((new Symbol()).name()));
		} else {
		    body = body.substituteVariable
			(qi.variable(),
			 new Function (new Symbol(), terms));
		}
	    }
	}

	// Don't need the quantifications any more!
	return body;
    }

    static public void debugPrintln(String s) {
	if (_debugPrint)
	    System.out.println(s);
    } 

    static public void debugPrint(String s) {
	if (_debugPrint)
	    System.out.print(s);
    }
} 
