package RRPC;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

public class Substitution implements java.io.Serializable {
	static final long serialVersionUID = -2229928639283703518L;
    // Map variables to terms Use strings as hash keys in case we have
    // different variable objects with the same name (in which case
    // they should be treated as the same variable.

    private Hashtable _table;

    public Substitution() {
	_table = new Hashtable();
    }

    public Object getBinding(Variable var) {
	return _table.get(var.toString());
    }

    public Object getBinding(String var) {
	return _table.get(var);
    }

    public boolean isBound(Variable var) {
	return (getBinding(var) != null);
    } 

    public boolean isEmpty() {
	return _table.isEmpty();
    }


    /**
     * extends this Substitution object with an association between the given
     * Variable and value.  Method changes the Substitution object.  Will not
     * work well for the various theorem provers which may want to share a
     * Substitution instance (or part thereof) among many lines of reasoning.
     */

    public Substitution addBinding(Variable var, Object val) {
	_table.put(var.toString(), val);
	return this;
    } 

    public String toString() {
	String result = "{";
	Enumeration vars = _table.keys();
	String var;
	Object val;

	if (vars.hasMoreElements()){
	    // Add first element
	    var = (String) vars.nextElement();
	    val = getBinding(var);
	    result = result + var + "/" + val;
	    
	    // Add the rest of the elements
	    while (vars.hasMoreElements()){
		var = (String) vars.nextElement();
		val = getBinding(var);
		result = result + ", " + var + "/" + val;
	    }
	}

	// Close and return
	result = result + "}";
	return result;
    }
}
