package RRPC;

import java.util.Vector;

public class TermList implements java.io.Serializable{
	static final long serialVersionUID = -3574639781796043191L;
    private Term _first;
    private TermList _theRest;

    public TermList (Term first, TermList _rest){
	_first = first;
	_theRest = _rest;
    }

    public Term getFirst() {
	return _first;
    }

    public TermList getRest() {
	return _theRest;
    }

    public int length() {
	int count = 0;
	TermList current = _theRest;

	if (_first != null)
	    count = 1;
	else
	    return count;

	while (current != null) {
	    count++;
	    current = current.getRest();
	}

	return count;
    } // end of METHOD length()

    /**
     * see Proposition.obtainVariables()
     *
     * get the variables in this list of terms
     */
    public Vector obtainVariables() {
	Vector result;
	int resultSize;
	// the variables inside _first
	Vector firstResult = _first.obtainVariables();

	if (_theRest == null)
	    return firstResult;

	result = _theRest.obtainVariables();

	if (result.size() == 0)
	    return firstResult;

	// we do it this way because otherwise result.size() would change in the
	// course of the for loops
	resultSize = result.size();
	// check to make sure the Variable wasn't already there before
	for (int i = 0; i < resultSize; i++) {
	    boolean found = false;
	    Term t = (Term)result.elementAt(i);
	    Term frt = null;

	    for (int j = 0; j < firstResult.size(); j++) {
		frt = (Term)firstResult.elementAt(j);
		if (frt.equals(t)) {
		    found = true;
		    break;
		}
	    }

	    if (!found && frt != null)
		result.addElement(frt);
	}

	return result;
    } // end of METHOD obtainVariables


    // Assumes t1 is equal to the length of this
    public Substitution unify(TermList tl, Substitution s) {
	Substitution newS = _first.unify(tl.getFirst(), s);
	if (newS == null){
	    // The first two didn't unify
	    return null;
	} else if (_theRest == null) {
	    // We're done!
	    return newS;
	} else {
	    // Do the rest
	    return _theRest.unify(tl.getRest(), newS);
	}
    } 

    public TermList substituteVariable (Variable oldVar, Term newVar){
	if (_theRest == null){
	    return new TermList (_first.substituteVariable (oldVar, newVar), null);
	}
	else {
	    return new TermList (_first.substituteVariable (oldVar, newVar),
				 _theRest.substituteVariable (oldVar, newVar));
	}
    } 

    public TermList substitute (Substitution s) {
	if (_theRest == null) {
	    return new TermList (_first.substitute(s), null);
	}
	else {
	    return new TermList (_first.substitute(s),
				 _theRest.substitute(s));
	}
    } 

    // Convert free vars to constants
    public TermList removeQuantifiers (Vector quantifications){
	if (_theRest == null){
	    return new TermList (_first.removeQuantifiers (quantifications), 
				 null);
	} else {
	    return new TermList (_first.removeQuantifiers (quantifications),
				 _theRest.removeQuantifiers (quantifications));
	}
    }

    public String toString(){
	if (_theRest == null){
	    return _first.toString();
	} else {
	    return _first + ", " + _theRest;
	}
    }

    // Return true if each of these terms subsumes or is equal to its
    // counterpart in the argument.  A constant subsumes a variable
    // Assumes the same length.
    
    public boolean subsume (TermList terms, Substitution s){
	Substitution newS = _first.subsumes(terms.getFirst(), s);
	if (newS == null){
	    // The first two didn't subsume
	    return false;
	} else {
	    return (_theRest == null ||
		    _theRest.subsume(terms.getRest(), newS));
	}
    }

    public void substituteTerm(Term newTerm, Term oldTerm){
	if (_first == null){
	    return;
	}

	if (_first == oldTerm){
	    _first = newTerm;
	} else {
	    // Try the subcomponents 
	    _first.substituteTerm(newTerm, oldTerm);
	}
	
	if (_theRest != null){
	    _theRest.substituteTerm(newTerm, oldTerm);
	}
    }

    // Return a vector of terms contained in this proposition that unify with t
    public Vector getPossibleMatches(Term t){
	Vector result = new Vector();

	if (_first == null){
	    return result;
	}

	// First, try the first element
	Substitution s = _first.unify(t, new Substitution());
	if (s != null){
	    result.addElement(_first);
	}

	// Now, subcomponents of the first
	result.addAll(_first.getPossibleMatches(t));

	// And the rest
	if (_theRest != null){
	    result.addAll(_theRest.getPossibleMatches(t));
	}

	return result;

    }

    public TermList deepCopy(){
	if (_first == null){
	    return this;
	}
	Term firstCopy = _first.deepCopy();

	TermList restCopy = null;
	if (_theRest != null){
	    restCopy = _theRest.deepCopy();
	}
	return new TermList(firstCopy, restCopy);
    }

    // Use string equality

    public TermList substituteTermAndCopy(Term newTerm, Term oldTerm){
	if (_first == null){
	    return this;
	}

	Term firstCopy;
        if (_first.toString().equals(oldTerm.toString())){
	    firstCopy = newTerm;
	} else {
	    firstCopy = _first.substituteTermAndCopy(newTerm, oldTerm);
	}

	TermList restCopy = null;
	if (_theRest != null){
	    restCopy = _theRest.substituteTermAndCopy(newTerm, oldTerm);
	}
	return new TermList(firstCopy, restCopy);
    }

} 
